#ifndef CPPUNIT_H__QM
#define CPPUNIT_H__QM

#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestResult.h>
#include <cppunit/extensions/Orthodox.h>
#include <cppunit/Protector.h>
#include <cppunit/TestComposite.h>
#include <cppunit/extensions/TestSetup.h>
#include <cppunit/extensions/TestCaseDecorator.h>
#include "cppunit/extensions/RepeatedTest.h"

class SimpleTestWithName : public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( SimpleTestWithName );
		CPPUNIT_TEST( testOk );
	CPPUNIT_TEST_SUITE_END();

public:
	void testOk();
};

class FixtureTest : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE( FixtureTest );
	CPPUNIT_TEST( testOK );
	CPPUNIT_TEST( testSetupCalled );
	CPPUNIT_TEST_SUITE_END();

	bool setUpDone;

public:
	FixtureTest(): setUpDone(false) {}

	void setUp();
	void tearDown();

	void testOK();
	void testSetupCalled();
};


class Exception : public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( Exception );
		CPPUNIT_TEST_EXCEPTION( testThrowOk, int );
	CPPUNIT_TEST_SUITE_END();

public:
	void testThrowOk();
};

class CustomExceptionChecker: public CPPUNIT_NS::ExceptionTestCaseDecorator<int>
{
public:
	CustomExceptionChecker( TestCase *test )
		: ExceptionTestCaseDecorator<int>(test)
	{
	}

protected:
	virtual void checkException( int &e )
	{
		CPPUNIT_ASSERT_EQUAL(e, 1);
	}
};

#define CPPUNIT_TEST_EXCEPTION_IS_1(testMethod, ExcType) \
	CPPUNIT_TEST_SUITE_ADD_TEST(new CustomExceptionChecker(new CPPUNIT_NS::TestCaller<CustomExceptionCheckerTest>(context.getTestNameFor(#testMethod), &testMethod)))

class CustomExceptionCheckerTest : public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( CustomExceptionCheckerTest );
		CPPUNIT_TEST_EXCEPTION_IS_1( testThrowOk, int );
	CPPUNIT_TEST_SUITE_END();

public:

	void testThrowOk();
};

// Tests for derived fixtures:

class FixtureBase : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE( FixtureBase );
	CPPUNIT_TEST( testA );
	CPPUNIT_TEST_SUITE_END();

protected:
	int a;

public:
	virtual void setUp();
	virtual void tearDown();

	void testA();
};

class FixtureDerived: public FixtureBase
{
	CPPUNIT_TEST_SUB_SUITE( FixtureDerived, FixtureBase );
	CPPUNIT_TEST( testB );
	CPPUNIT_TEST_SUITE_END();

	int b;

public:
	virtual void setUp();
	virtual void tearDown();

	void testB();
};

// Tests for abstract test
class AbstractBase : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE( AbstractBase );
	CPPUNIT_TEST( testA );
	CPPUNIT_TEST_SUITE_END_ABSTRACT(); // notice different end macro!

protected:
	int a;

public:
	virtual void setUp();
	virtual void tearDown();

	void testA();
};

class DerivedFromAbstract: public AbstractBase
{
	CPPUNIT_TEST_SUB_SUITE( DerivedFromAbstract, AbstractBase );
	CPPUNIT_TEST( testB );
	CPPUNIT_TEST_SUITE_END();

	int b;

public:
	virtual void setUp();
	virtual void tearDown();

	void testB();
};



// Test for template test cases:

template <typename T>
class TemplateTest : public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( TemplateTest );
	CPPUNIT_TEST( testSizeOf );
	CPPUNIT_TEST_SUITE_END();

public:

	void testSizeOf();
};



// Test for custom TestCallers

template <class T>
class CustomTestCaller: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef void (T::*TestMethod)();
	typedef CPPUNIT_NS::TestCaller<T> Parent;

	CustomTestCaller(const std::string &testname, TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		active = true;
		Parent::run(result);
		active = false;
	}

	static bool active;
};

template <class T>
bool CustomTestCaller<T>::active = false;

class TestCaller : public CPPUNIT_NS::TestCase
{
	typedef void (TestCaller::*TestMethod)();
	CPPUNIT_TEST_SUITE( TestCaller );
	CPPUNIT_TEST_SUITE_ADD_TEST(new CustomTestCaller<TestCaller>("TestCaller::mp", &testCallerIsUsed));
	CPPUNIT_TEST_SUITE_END();

public:

	void testCallerIsUsed();
};



// Tests that throw Exception

class ThrowException : public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( ThrowException );
	CPPUNIT_TEST(throwException);
	CPPUNIT_TEST_SUITE_END();

public:

	void throwException();
};



// Tests for protectors

class CustomProtector1: public CPPUNIT_NS::Protector
{
public:
	virtual bool protect( const CPPUNIT_NS::Functor &functor,
		const CPPUNIT_NS::ProtectorContext &context )
	{
		active = true;
		functor();
		active = false;

		return true;
	}

	static bool active;
};

class CustomProtector2: public CPPUNIT_NS::Protector
{
public:
	virtual bool protect( const CPPUNIT_NS::Functor &functor,
		const CPPUNIT_NS::ProtectorContext &context )
	{
		active = true;
		functor();
		active = false;

		return true;
	}

	static bool active;
};

template <class T, class Protector>
class CustomTestCallerWithProtector: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;
	typedef void (T::*TestMethod)();
	CustomTestCallerWithProtector(const std::string &testname, TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		result->pushProtector(new Protector);
		Parent::run(result);
		result->popProtector();
	}
};

template <class T, class Protector>
class CustomTestCallerWithProtectorGuard: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;
	typedef void (T::*TestMethod)();
	CustomTestCallerWithProtectorGuard(const std::string &testname, TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		CPPUNIT_NS::ProtectorGuard protectorGuard(result, new Protector);
		Parent::run(result);
	}
};

template <class T>
class CustomTestCallerWithProtector1And2: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;
	typedef void (T::*TestMethod)();
	CustomTestCallerWithProtector1And2(const std::string &testname, TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		result->pushProtector(new CustomProtector1);
		result->pushProtector(new CustomProtector2);
		Parent::run(result);
		result->popProtector();
		result->popProtector();
	}
};

class CustomProtectorTest : public CPPUNIT_NS::TestFixture
{
  typedef CustomTestCallerWithProtector<CustomProtectorTest, CustomProtector1>
	  CustomTestCallerWithProtector1;
  typedef CustomTestCallerWithProtectorGuard<CustomProtectorTest, CustomProtector1>
	  CustomTestCallerWithProtectorGuard1;

  CPPUNIT_TEST_SUITE( CustomProtectorTest );
  CPPUNIT_TEST(testNotActive);
  CPPUNIT_TEST_SUITE_ADD_TEST(new CustomTestCallerWithProtector1("CustomProtectorTest::test2", &testFirstActive));
  CPPUNIT_TEST_SUITE_ADD_TEST(new CustomTestCallerWithProtectorGuard1("CustomProtectorTest::test3", &testFirstActiveWithGuard));
  CPPUNIT_TEST_SUITE_ADD_TEST(new CustomTestCallerWithProtector1And2<CustomProtectorTest>("CustomProtectorTest::test1", &testBothActive));
  CPPUNIT_TEST_SUITE_END();

public:

  void testNotActive();
  void testFirstActive();
  void testFirstActiveWithGuard();
  void testBothActive();
};



// Tests for protector that reports exception

class CustomProtectorWithException: public CPPUNIT_NS::Protector
{
	int count;
public:
	CustomProtectorWithException(): count(0)
	{}

	virtual bool protect( const CPPUNIT_NS::Functor &functor,
		const CPPUNIT_NS::ProtectorContext &context );
};

class CustomProtectorWithExceptionTest: public CPPUNIT_NS::TestFixture
{
  typedef CustomTestCallerWithProtector<CustomProtectorWithExceptionTest, CustomProtectorWithException>
	  CustomTestCallerWithProtectorWithException;
  CPPUNIT_TEST_SUITE( CustomProtectorWithExceptionTest);
  CPPUNIT_TEST_SUITE_ADD_TEST(
	  new CustomTestCallerWithProtectorWithException("CustomProtectorWithExceptionTest::test", &test)
	  );
  CPPUNIT_TEST_SUITE_END();

public:

  void test();
};



// Test for custom TestCaller that call addError and addFailure

template <class T>
class CustomTestCallerWithFailure: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;

	CustomTestCallerWithFailure(const std::string &testname, CPPUNIT_NS::TestCaller<T>::TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result);
};

template <class T>
class CustomTestCallerWithException: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;

	CustomTestCallerWithException(const std::string &testname, CPPUNIT_NS::TestCaller<T>::TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result);
};


// Tests for CPPUNIT_TEST_FAIL

class FailTest: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( FailTest );
		CPPUNIT_TEST_FAIL(testFail_Ok);
	CPPUNIT_TEST_SUITE_END();

public:

	void testFail_Ok();
};


// Tests for CPPUNIT_TEST_SUITE_ADD_CUSTOM_TESTS

template <class T>
class CustomTestCallerWithFixture: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;
	typedef void (T::*TestMethod)();

	CustomTestCallerWithFixture(const std::string &name,
		TestMethod testMethod,
		T *fixture):

		CPPUNIT_NS::TestCaller<T>(name, testMethod, fixture)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		active = true;
		Parent::run(result);
		active = false;
	}

	static bool active;
};

template <class T>
bool CustomTestCallerWithFixture<T>::active = false;

class AddCustomTests: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( AddCustomTests );
	CPPUNIT_TEST_SUITE_ADD_CUSTOM_TESTS(addTests);
	CPPUNIT_TEST_SUITE_END();

	static void addTests(TestSuiteBuilderContextType &context)
	{
		context.addTest( new CustomTestCallerWithFixture<AddCustomTests>( context.getTestNameFor( "testA" ),
											 &AddCustomTests::test1,
											 context.makeFixture()));
		context.addTest( new CustomTestCallerWithFixture<AddCustomTests>( context.getTestNameFor( "testB" ),
											 &AddCustomTests::test2,
											 context.makeFixture()));
	}
public:

	void test1();
	void test2();
};


// Tests for test suite properties
/*
class Properties: public CPPUNIT_NS::TestCase
{
	std::string m_param;

	CPPUNIT_TEST_SUITE( Properties );
	CPPUNIT_TEST(testEmptyProperties);
	CPPUNIT_TEST(testOneProperty);
	CPPUNIT_TEST(testTwoProperties);
	CPPUNIT_TEST(testOverrideProperty);
	CPPUNIT_TEST(testOverrideTwoProperties);
	CPPUNIT_TEST_SUITE_END();

	CPPUNIT_NS::TestNamer m_testNamer;
public:
	Properties();

	void testEmptyProperties();
	void testOneProperty();
	void testTwoProperties();
	void testOverrideProperty();
	void testOverrideTwoProperties();
};
*/


// Tests for CPPUNIT_TEST_SUITE_PROPERTY

template <class T>
class CustomTestCallerWithString: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;
	typedef void (T::*TestMethod)();

	CustomTestCallerWithString(const std::string &name,
		TestMethod testMethod,
		T *fixture,
		const std::string &param):

		CPPUNIT_NS::TestCaller<T>(name, testMethod, fixture)
	{
		fixture->SetParam(param);
	}
};

class UseProperties: public CPPUNIT_NS::TestCase
{
	std::string m_param;

	CPPUNIT_TEST_SUITE( UseProperties );
	CPPUNIT_TEST_SUITE_PROPERTY("par", "abc");
	CPPUNIT_TEST_SUITE_ADD_TEST(
		new CustomTestCallerWithString<UseProperties>
			("UseProperties::test1", &test1, context.makeFixture(), context.getStringProperty("par"))
	);
	CPPUNIT_TEST_SUITE_END();

public:

	void test1();

	void SetParam(const std::string &param)
	{
		m_param = param;
	}
};


// Tests for TestDecorator:

class MyTestDecorator: public CPPUNIT_NS::TestDecorator
{
public:
	typedef CPPUNIT_NS::TestDecorator Parent;

	MyTestDecorator(CPPUNIT_NS::Test *test):
		CPPUNIT_NS::TestDecorator(test)
	{}

	static int beforeCalled, afterCalled;

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		++beforeCalled;
		Parent::run(result);
		++afterCalled;
	}
};

class TestDecorator: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( TestDecorator );
	CPPUNIT_NS::Test *testa = new CPPUNIT_NS::TestCaller<TestDecorator>(context.getTestNameFor("test1"), &test1);
	CPPUNIT_TEST_SUITE_ADD_TEST(new MyTestDecorator(testa));
	CPPUNIT_NS::Test *testb = new CPPUNIT_NS::TestCaller<TestDecorator>(context.getTestNameFor("test2"), &test2);
	CPPUNIT_TEST_SUITE_ADD_TEST(new MyTestDecorator(testb));
	CPPUNIT_TEST_SUITE_END();

public:
	void test1();
	void test2();
};


// Tests for TestCaseDecorator:

class MyTestCaseDecorator: public CPPUNIT_NS::TestCaseDecorator
{
public:
	typedef CPPUNIT_NS::TestCaseDecorator Parent;

	MyTestCaseDecorator(CPPUNIT_NS::TestCase *testCase):
		CPPUNIT_NS::TestCaseDecorator(testCase)
	{}

	static int setUpCalled, runCalled, tearDownCalled;

	virtual void setUp()
	{
		++setUpCalled;
		Parent::setUp();
	}

	virtual void tearDown()
	{
		++tearDownCalled;
		Parent::tearDown();
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		++runCalled;
		Parent::run(result);
	}
};

class TestCaseDecorator: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( TestCaseDecorator );
	CPPUNIT_NS::TestCase *testa = new CPPUNIT_NS::TestCaller<TestCaseDecorator>(context.getTestNameFor("test1"), &test1);
	CPPUNIT_TEST_SUITE_ADD_TEST(new MyTestCaseDecorator(testa));
	CPPUNIT_NS::TestCase *testb = new CPPUNIT_NS::TestCaller<TestCaseDecorator>(context.getTestNameFor("test2"), &test2);
	CPPUNIT_TEST_SUITE_ADD_TEST(new MyTestCaseDecorator(testb));
	CPPUNIT_TEST_SUITE_END();

public:
	static int setUpCalled, tearDownCalled;

	virtual void setUp()
	{
		++setUpCalled;
	}
	virtual void tearDown()
	{
		++tearDownCalled;
	}

	void test1();
	void test2();
};


// Tests for RepeatedTest:

template <class T>
class CustomTestCallerThatStops: public CPPUNIT_NS::TestCaller<T>
{
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;
	typedef void (T::*TestMethod)();

	CustomTestCallerThatStops(const std::string &testname, TestMethod testMethod):
		CPPUNIT_NS::TestCaller<T>(testname, testMethod)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result)
	{
		Parent::run(result);
		result->stop();
	}
};

class RepeatedTest: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( RepeatedTest );
	CPPUNIT_NS::Test *test_1 = new CPPUNIT_NS::TestCaller<RepeatedTest>(context.getTestNameFor("test1"), &test1);
	CPPUNIT_TEST_SUITE_ADD_TEST(new CPPUNIT_NS::RepeatedTest(test_1, 3));
	CPPUNIT_NS::Test *test_2 = new CustomTestCallerThatStops<RepeatedTest>(context.getTestNameFor("test1"), &test2);
	CPPUNIT_TEST_SUITE_ADD_TEST(new CPPUNIT_NS::RepeatedTest(test_2, 3));
	CPPUNIT_TEST_SUITE_END();

public:
	void test1();
	void test2();
};


// Tests for TestSetUp:

class MyTestSetUp: public CPPUNIT_NS::TestSetUp
{
public:
	MyTestSetUp(CPPUNIT_NS::Test *test):
		CPPUNIT_NS::TestSetUp(test)
	{}

	static int setUpCalled, tearDownCalled;

	virtual void setUp()
	{
		++setUpCalled;
	}

	virtual void tearDown()
	{
		++tearDownCalled;
	}
};

class TestSetUp: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( TestSetUp );
	CPPUNIT_NS::Test *testa = new CPPUNIT_NS::TestCaller<TestSetUp>(context.getTestNameFor("test1"), &test1);
	CPPUNIT_TEST_SUITE_ADD_TEST(new MyTestSetUp(testa));
	CPPUNIT_NS::Test *testb = new CPPUNIT_NS::TestCaller<TestSetUp>(context.getTestNameFor("test2"), &test2);
	CPPUNIT_TEST_SUITE_ADD_TEST(new MyTestSetUp(testb));
	CPPUNIT_TEST_SUITE_END();

public:
	void test1();
	void test2();
};


// tests for Orthodoxy:

struct Orthodox_Ok
{
	Orthodox_Ok(): m_a(0) {}
	Orthodox_Ok(int a): m_a(a) {}

	int m_a;

	bool operator==(const Orthodox_Ok& other) const
	{
		return m_a == other.m_a;
	}
	bool operator!=(const Orthodox_Ok& other) const
	{
		return m_a != other.m_a;
	}
	Orthodox_Ok operator!() const
	{
		return Orthodox_Ok(1-m_a);
	}
};

struct Orthodox_EqualsWrong
{
	Orthodox_EqualsWrong(): m_a(0) {}
	Orthodox_EqualsWrong(int a): m_a(a) {}

	int m_a;

	bool operator==(const Orthodox_EqualsWrong& other) const
	{
		return m_a != other.m_a;
	}
	bool operator!=(const Orthodox_EqualsWrong& other) const
	{
		return m_a != other.m_a;
	}
	Orthodox_EqualsWrong operator!() const
	{
		return Orthodox_EqualsWrong(1-m_a);
	}
};

struct Orthodox_DiffWrong
{
	Orthodox_DiffWrong(): m_a(0) {}
	Orthodox_DiffWrong(int a): m_a(a) {}

	int m_a;

	bool operator==(const Orthodox_DiffWrong& other) const
	{
		return m_a == other.m_a;
	}
	bool operator!=(const Orthodox_DiffWrong& other) const
	{
		return m_a == other.m_a;
	}
	Orthodox_DiffWrong operator!() const
	{
		return Orthodox_DiffWrong(1-m_a);
	}
};

struct Orthodox_NegateWrong
{
	Orthodox_NegateWrong(): m_a(0) {}
	Orthodox_NegateWrong(int a): m_a(a) {}

	int m_a;

	bool operator==(const Orthodox_NegateWrong& other) const
	{
		return m_a == other.m_a;
	}
	bool operator!=(const Orthodox_NegateWrong& other) const
	{
		return m_a != other.m_a;
	}
	Orthodox_NegateWrong operator!() const
	{
		return Orthodox_NegateWrong(m_a);
	}
};

struct Orthodox_CopyConstructorWrong
{
	Orthodox_CopyConstructorWrong(): m_a(0) {}
	Orthodox_CopyConstructorWrong(int a): m_a(a) {}
	Orthodox_CopyConstructorWrong(const Orthodox_CopyConstructorWrong &other)
		: m_a(1+other.m_a)
	{}

	int m_a;

	bool operator==(const Orthodox_CopyConstructorWrong& other) const
	{
		return m_a == other.m_a;
	}
	bool operator!=(const Orthodox_CopyConstructorWrong& other) const
	{
		return m_a != other.m_a;
	}
	Orthodox_CopyConstructorWrong operator!() const
	{
		return Orthodox_CopyConstructorWrong(1-m_a);
	}
};

struct Orthodox_AssignWrong
{
	Orthodox_AssignWrong(): m_a(0) {}
	Orthodox_AssignWrong(int a): m_a(a) {}

	int m_a;

	Orthodox_AssignWrong& operator=(const Orthodox_AssignWrong& other)
	{
		m_a = 1+other.m_a;
		return *this;
	}
	bool operator==(const Orthodox_AssignWrong& other) const
	{
		return m_a == other.m_a;
	}
	bool operator!=(const Orthodox_AssignWrong& other) const
	{
		return m_a != other.m_a;
	}
	Orthodox_AssignWrong operator!() const
	{
		return Orthodox_AssignWrong(1-m_a);
	}
};


class Orthodoxy_Ok: public CPPUNIT_NS::TestCase
{
	CPPUNIT_TEST_SUITE( Orthodoxy_Ok );
		CPPUNIT_TEST_SUITE_ADD_TEST(new CPPUNIT_NS::Orthodox<Orthodox_Ok>);
	CPPUNIT_TEST_SUITE_END();
};

template <class T>
class CustomTestCallerForComposite: public CPPUNIT_NS::TestCaller<T>
{
	bool m_Fail;
public:
	typedef CPPUNIT_NS::TestCaller<T> Parent;

	CustomTestCallerForComposite(const std::string &testname, CPPUNIT_NS::TestCaller<T>::TestMethod testMethod, bool Fail)
		: CPPUNIT_NS::TestCaller<T>(testname, testMethod)
		, m_Fail(Fail)
	{
	}

	virtual void run(CPPUNIT_NS::TestResult *result);
};

class CompositeTest: public CPPUNIT_NS::TestCase
{
	class Composite: public CPPUNIT_NS::TestComposite
	{
		CPPUNIT_NS::Test* m_Test1;
		CPPUNIT_NS::Test* m_Test2;
	public:
		Composite()
			: CPPUNIT_NS::TestComposite("MyComposite")
		{
			m_Test1 = new CPPUNIT_NS::TestCaller<CompositeTest>("Test1", &CompositeTest::test1);
			m_Test2 = new CPPUNIT_NS::TestCaller<CompositeTest>("Test2", &CompositeTest::test2);
		}

		int getChildTestCount(void) const
		{
			return 2;
		}

		virtual Test *doGetChildTestAt( int index ) const
		{
			if (index==0)
				return m_Test1;
			else
				return m_Test2;
		}

		void setUp()
		{}

		void tearDown()
		{}
	};

	CPPUNIT_TEST_SUITE( CompositeTest);
		CPPUNIT_TEST_SUITE_ADD_TEST(new Composite);
	CPPUNIT_TEST_SUITE_END();

	void test1();
	void test2();
};

#endif // #ifndef CPPUNIT_H__QM
