//  Copyright (C) 2006-2009 Q-Mentum.
//  See the file license.txt for limitations on usage of this code.

//  For more information about Q-Mentum, visit http://www.q-mentum.com

#include "uquonitest/Test.hpp"
#include "uquonitest/TestWrapper.hpp"
#include "richbool/richbool.hpp"

#include "ByteSize.hpp"

// Three more advanced features are demonstrated here:
// - dealing with memory leaks caused by the initialization of a singleton
// - creating a testwrapper to catch exceptions
// - checking if a statement throws an exception


// Define a class to catch exceptions in the tests in this group:
class ByteSizeExceptionCatcher: public UquoniTest::ExceptionCatcher
{
public:
	virtual void Run(UquoniTest::TestObject *testObject,
		UquoniTest::TestResult *testResult,
		UquoniTest::TestData *data,
		UquoniTest::TestWrapperContext *context)
	{
		try
		{
			RunNext(testObject, testResult, data, context);
		}
		catch (ByteSizeException &exc)
		{
			UquoniTest::OnException(testObject, testResult,
				"ByteSizeException",
				exc.GetCause().c_str());
		}
	}

	virtual const char* GetName() const
	{
		return "ByteSizeExceptionCatcher";
	}

	virtual UquoniTest::TestWrapper* CloneTW() const
	{
		return new ByteSizeExceptionCatcher;
	}
};

uqtTEST_GROUP(ByteSize)
{
	// Create an instance of the class ByteSizeExceptionCatcher,
	// and give a pointer to it to uqtADD_TEST_WRAPPER.
	// This is done inside the group, so it is only used for tests in this group.
	static ByteSizeExceptionCatcher byteSizeExceptionCatcher;
	uqtADD_TEST_WRAPPER(&byteSizeExceptionCatcher);


	// The class ByteSize uses a stringstream object,
	// which initializes an object the first time the class is used.
	// This allocates memory, which looks like a memoryleak but isn't really.
	// Therefore we add some code with the macro uqtCAUSE_MEMORY_LEAK
	// to let this initialization take place, of which UquoniTest will ignore
	// the 'memory leak'.
	uqtCAUSE_MEMORY_LEAK
	{
		ByteSize b;
		b.SetContent("1 byte");
		unsigned int nrBytes = b();
	}


	///////////////////////////////////////////////////////
	// The tests:

	uqtTEST(OneByte)
	{
		ByteSize b;
		b.SetContent("1 byte");

		uqtASSERT(rbEQUAL(b(), 1));
	}

	uqtTEST(TwoBytes)
	{
		ByteSize b;
		b.SetContent("2 byte");

		uqtASSERT(rbEQUAL(b(), 2));
	}

	uqtTEST(OneByteAndSpace)
	{
		ByteSize b;
		b.SetContent(" 1 byte");

		uqtASSERT(rbEQUAL(b(), 1));
	}

	uqtTEST(OneKiloByte)
	{
		ByteSize b;
		b.SetContent("1 kB");

		uqtASSERT(rbEQUAL(b(), 1000));
	}

	uqtTEST(OneAndAHalfKiloByte)
	{
		ByteSize b;
		b.SetContent("1.5 kB");

		uqtASSERT(rbEQUAL(b(), 1500));
	}

	uqtTEST(OneAndAHalfKibiByte)
	{
		ByteSize b;
		b.SetContent("1.5 KiB");

		uqtASSERT(rbEQUAL(b(), 1536));
	}

	uqtTEST(OneAndAHalfMegaByte)
	{
		ByteSize b;
		b.SetContent("1.5 MB");

		uqtASSERT(rbEQUAL(b(), 1500000));
	}

	uqtTEST(ExceptionThrownIfWrongUnitGiven)
	{
		ByteSize b;
		uqtASSERT_THROWS(b.SetContent("1.5 wrong"), ByteSizeException);
	}
}
